@@ -196,11 +196,20 @@ void TestCompletion::privateStaticObjectCompletion()
 
     DUContext* funContext = top->childContexts().first()->localDeclarations().first()->internalContext();
 
+    {
     PhpCompletionTester tester(funContext, "self::");
 
     QCOMPARE(tester.completionContext->memberAccessOperation(), CodeCompletionContext::StaticMemberAccess);
 
     QCOMPARE(tester.names, QStringList() << "spubf" << "$spub" << "c" << "sprotf" << "$sprot" << "sprivf" << "$spriv");
+    }
+    {
+    PhpCompletionTester tester(funContext, "static::");
+
+    QCOMPARE(tester.completionContext->memberAccessOperation(), CodeCompletionContext::StaticMemberAccess);
+
+    QCOMPARE(tester.names, QStringList() << "spubf" << "$spub" << "c" << "sprotf" << "$sprot" << "sprivf" << "$spriv");
+    }
 }
 void TestCompletion::protectedObjectCompletion()
 {
@@ -306,6 +315,42 @@ void TestCompletion::functionCall()
     QVERIFY(searchDeclaration(tester.items, top->localDeclarations().at(1)));
 }
 
+void TestCompletion::nestedFunctionCall_data()
+{
+    QTest::addColumn<QString>("text");
+
+    QTest::newRow("nested") << QString("bar(foo(");
+    QTest::newRow("nested prev arg") << QString("bar(1, foo(");
+    QTest::newRow("nested prev func call") << QString("bar(foo(1), foo(");
+    QTest::newRow("nested prev arg comma") << QString("bar(1, bar(1, ");
+    QTest::newRow("nested prev func comma") << QString("bar(1, bar(foo(1), ");
+}
+
+void TestCompletion::nestedFunctionCall()
+{
+    QFETCH(QString, text);
+
+    //                 0         1         2         3         4         5         6         7
+    //                 01234567890123456789012345678901234567890123456789012345678901234567890123456789
+    QByteArray method("<? function foo($a) {return 1;} function bar($b, $c) {return 2;}");
+
+    TopDUContext* top = parse(method, DumpNone);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock;
+
+    PhpCompletionTester tester(top, text);
+    QVERIFY(tester.completionContext->parentContext());
+    QVERIFY(tester.completionContext->parentContext()->parentContext());
+    QVERIFY(!tester.completionContext->parentContext()->parentContext()->parentContext());
+    QCOMPARE(tester.completionContext->parentContext()->memberAccessOperation(),
+             CodeCompletionContext::FunctionCallAccess);
+    QCOMPARE(tester.completionContext->parentContext()->parentContext()->memberAccessOperation(),
+             CodeCompletionContext::FunctionCallAccess);
+
+    QVERIFY(searchDeclaration(tester.items, top->localDeclarations().at(0)));
+    QVERIFY(searchDeclaration(tester.items, top->localDeclarations().at(1)));
+}
+
 void TestCompletion::newObjectFromOtherFile()
 {
 
@@ -687,7 +732,7 @@ void TestCompletion::overrideMethods()
 
 void TestCompletion::inArray()
 {
-    TopDUContext* top = parse("", DumpAll);
+    TopDUContext* top = parse("", DumpNone);
     DUChainReleaser releaseTop(top);
     DUChainWriteLocker lock(DUChain::lock());
 
@@ -921,37 +966,26 @@ void TestCompletion::nonGlobalInFunction()
     QVERIFY(!searchDeclaration(tester.items, decs.first()));
 }
 
-void TestCompletion::fileCompletion_data()
-{
-    QTest::addColumn<QString>("code");
-
-    QTest::newRow("include-quote") << QString("include \"");
-    QTest::newRow("include_once-quote") << QString("include_once \"");
-    QTest::newRow("require-quote") << QString("require \"");
-    QTest::newRow("require_once-quote") << QString("require_once \"");
-    QTest::newRow("include-paren") << QString("include ( \"");
-    QTest::newRow("include-quote2") << QString("include '");
-    QTest::newRow("include-paren2") << QString("include ( '");
-    QTest::newRow("include-dirname") << QString("include dirname(__FILE__) . \"/");
-    QTest::newRow("include-dirname2") << QString("include dirname(__FILE__) . '/");
-    QTest::newRow("include-dirname3") << QString("include ( dirname(__FILE__) . \"/");
-    QTest::newRow("include-dirname4") << QString("include ( dirname(__FILE__) . '/");
-    /** TODO:  "include __DIR__ . \"/" */
-}
-
 void TestCompletion::fileCompletion()
 {
     TopDUContext* top = parse("<?php ", DumpNone);
     DUChainReleaser releaseTop(top);
     DUChainWriteLocker lock(DUChain::lock());
 
-    QFETCH(QString, code);
-
     ///TODO: somehow offer files and check whether they work with relative sub-paths
     ///TODO: make sure items after dirname(__FILE__) or similar start with a /
-    kDebug() << code;
-    PhpCompletionTester tester(top, code);
-    QCOMPARE(tester.completionContext->memberAccessOperation(), CodeCompletionContext::FileChoose);
+    foreach ( const QString& code, QStringList() << "include \"" << "include_once \"" << "require_once \""
+                                                 << "require \"" << "include ( \""
+                                                 << "include dirname(__FILE__) . \"/"
+                                                 << "include ( dirname(__FILE__) . \"/"
+                                                 << "include '" << "include ( '"
+                                                 << "include ( dirname(__FILE__) . '/"
+                                                 /** TODO:  << "include __DIR__ . \"/" */ )
+    {
+        kDebug() << code;
+        PhpCompletionTester tester(top, code);
+        QCOMPARE(tester.completionContext->memberAccessOperation(), CodeCompletionContext::FileChoose);
+    }
 }
 
 void TestCompletion::instanceof()
@@ -1186,6 +1220,111 @@ void TestCompletion::funcCallInConditional()
     }
 }
 
+void TestCompletion::namespaces()
+{
+    //                 0         1         2         3         4         5         6         7
+    //                 01234567890123456789012345678901234567890123456789012345678901234567890123456789
+    QByteArray method("<? namespace foo\\bar {}\n");
+
+    TopDUContext* top = parse(method, DumpNone);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock;
+
+    {
+        PhpCompletionTester tester(top, "namespace ");
+        QCOMPARE(tester.completionContext->memberAccessOperation(), CodeCompletionContext::NamespaceChoose);
+        QVERIFY(!tester.completionContext->parentContext());
+
+        QCOMPARE(tester.names, QStringList() << "foo");
+    }
+}
+
+void TestCompletion::inNamespace()
+{
+    //                 0         1         2         3         4         5         6         7
+    //                 01234567890123456789012345678901234567890123456789012345678901234567890123456789
+    QByteArray method("<? namespace foo { function bar() {} }\n"
+                      "   namespace yxc { function qwe() {} }\n" );
+
+    TopDUContext* top = parse(method, DumpNone);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock;
+
+    {
+        PhpCompletionTester tester(top->childContexts().at(0), "");
+        QCOMPARE(tester.completionContext->memberAccessOperation(), CodeCompletionContext::NoMemberAccess);
+        QVERIFY(!tester.completionContext->parentContext());
+
+        QVERIFY(searchDeclaration(tester.items, top->localDeclarations().first()));
+        QVERIFY(searchDeclaration(tester.items, top->childContexts().first()->localDeclarations().first()));
+        QVERIFY(searchDeclaration(tester.items, top->localDeclarations().last()));
+        QVERIFY(!searchDeclaration(tester.items, top->childContexts().last()->localDeclarations().first()));
+    }
+    {
+        PhpCompletionTester tester(top->childContexts().at(0), "\\");
+        QCOMPARE(tester.completionContext->memberAccessOperation(), CodeCompletionContext::BackslashAccess);
+        QVERIFY(!tester.completionContext->parentContext());
+
+        QCOMPARE(tester.items.count(), 2);
+        QVERIFY(searchDeclaration(tester.items, top->localDeclarations().first()));
+        QVERIFY(!searchDeclaration(tester.items, top->childContexts().first()->localDeclarations().first()));
+        QVERIFY(searchDeclaration(tester.items, top->localDeclarations().last()));
+        QVERIFY(!searchDeclaration(tester.items, top->childContexts().last()->localDeclarations().first()));
+    }
+    {
+        PhpCompletionTester tester(top->childContexts().at(0), "\\foo\\");
+        QCOMPARE(tester.completionContext->memberAccessOperation(), CodeCompletionContext::BackslashAccess);
+        QVERIFY(!tester.completionContext->parentContext());
+
+        QCOMPARE(tester.items.count(), 1);
+        QVERIFY(!searchDeclaration(tester.items, top->localDeclarations().first()));
+        QVERIFY(searchDeclaration(tester.items, top->childContexts().first()->localDeclarations().first()));
+        QVERIFY(!searchDeclaration(tester.items, top->localDeclarations().last()));
+        QVERIFY(!searchDeclaration(tester.items, top->childContexts().last()->localDeclarations().first()));
+    }
+    {
+        PhpCompletionTester tester(top->childContexts().at(0), "\\yxc\\");
+        QCOMPARE(tester.completionContext->memberAccessOperation(), CodeCompletionContext::BackslashAccess);
+        QVERIFY(!tester.completionContext->parentContext());
+
+        QCOMPARE(tester.items.count(), 1);
+        QVERIFY(!searchDeclaration(tester.items, top->localDeclarations().first()));
+        QVERIFY(!searchDeclaration(tester.items, top->childContexts().first()->localDeclarations().first()));
+        QVERIFY(!searchDeclaration(tester.items, top->localDeclarations().last()));
+        QVERIFY(searchDeclaration(tester.items, top->childContexts().last()->localDeclarations().first()));
+    }
+}
+
+void TestCompletion::closures()
+{
+    //                 0         1         2         3         4         5         6         7
+    //                 01234567890123456789012345678901234567890123456789012345678901234567890123456789
+    QByteArray method("<? $l = function($a) {};\n" );
+
+    TopDUContext* top = parse(method, DumpNone);
+    QVERIFY(top);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock;
+
+    Declaration* l = top->localDeclarations().first();
+    Declaration* c = top->localDeclarations().last();
+    {
+        PhpCompletionTester tester(top, "");
+        QVERIFY(tester.containsDeclaration(l));
+        QVERIFY(!tester.containsDeclaration(c));
+    }
+    {
+        PhpCompletionTester tester(top, "$l(");
+        QVERIFY(tester.containsDeclaration(l));
+        QVERIFY(!tester.containsDeclaration(c));
+
+        QVERIFY(tester.completionContext->parentContext());
+        QVERIFY(!tester.completionContext->parentContext()->parentContext());
+        QCOMPARE(tester.completionContext->parentContext()->memberAccessOperation(),
+                 CodeCompletionContext::FunctionCallAccess);
+    }
+}
+
 }
 
 #include "test_completion.moc"
