@@ -49,6 +49,8 @@ private slots:
     void privateStaticObjectCompletion();
     void methodCall();
     void functionCall();
+    void nestedFunctionCall_data();
+    void nestedFunctionCall();
     void newObjectFromOtherFile();
     void constantFromOtherFile();
     void baseClass();
@@ -77,7 +79,6 @@ private slots:
     void phpStartTag();
     void outsidePhpContext();
     void nonGlobalInFunction();
-    void fileCompletion_data();
     void fileCompletion();
     void instanceof();
     void afterFunctionArg();
@@ -90,6 +91,9 @@ private slots:
     void ctorCall();
     void chainedCalling();
     void funcCallInConditional();
+    void namespaces();
+    void inNamespace();
+    void closures();
 };
 
 }
